// Copyright 2025 RnD Center "ELVEES", JSC

#ifndef CONVOLUTION_SMALL_ASM_KER_H
#define CONVOLUTION_SMALL_ASM_KER_H

#include "elcore50-matrix-lib/common.h"

enum Conv_version
{
    VER_1x16x128,
    VER_4x4x128,
    VER_4x8x64,
    VER_8x8x32
};

typedef struct Tensor_fl32{
    float *data;
    int batch, height, width, channel;
} Tensor_fl32;

typedef struct Tensor_fl16{
    uint16_t *data;
    int batch, height, width, channel;
} Tensor_fl16;

typedef struct Weight_fl32{
    float *data;
    int kernelX, kernelY, srcC, dstC;
    int strideX, strideY, dilationX, dilationY;
} Weight_fl32;

typedef struct Weight_fl16{
    uint16_t *data;
    int kernelX, kernelY, srcC, dstC;
    int strideX, strideY, dilationX, dilationY;
} Weight_fl16;

typedef struct pads{
    int padX;
    int padY;
    int padW;
    int padH;
} pads;

typedef struct size_coef{
    int coef_src_h;
    int coef_src_w;
    int coef_src_ch;
    int coef_dst_ch;
} size_coef;

typedef struct strides{
    int strideX;
    int strideY;
} strides;

extern "C" void convolution_16x128_fl16_ver1(uint16_t* src,
                                             uint16_t* weight,
                                             int src_width_stride,
                                             int src_height_stride,
                                             int src_batch_stride,
                                             int weight_width_stride,
                                             int weight_height_stride,
                                             int kernelX,
                                             int kernelY,
                                             int stride_X_in_bytes,
                                             int stride_Y_in_bytes,
                                             int loop_srcC_kerX_iter_count,
                                             int32_t* tic_counter,
                                             int32_t* instr_counter); // srcC / 8 * kerX

extern "C" void convolution_16x128_fl16_ver1_with_offset(uint16_t* src,
                                                         uint16_t* weight,
                                                         int src_width_stride,
                                                         int src_height_stride,
                                                         int src_batch_stride,
                                                         int weight_width_stride,
                                                         int weight_height_stride,
                                                         int kernelX,
                                                         int kernelY,
                                                         int stride_X_in_bytes,
                                                         int stride_Y_in_bytes,
                                                         int loop_srcC_kerX_iter_count); // srcC / 8 * kerX

extern "C" void convolution_32x64_fl16_ver1(uint16_t* src,
                                            uint16_t* weight,
                                            int src_width_stride,
                                            int src_height_stride,
                                            int src_batch_stride,
                                            int weight_width_stride,
                                            int weight_height_stride,
                                            int kernelX,
                                            int kernelY,
                                            int stride_X_in_bytes,
                                            int stride_Y_in_bytes,
                                            int loop_srcC_kerX_iter_count); // srcC / 4 * kerX

extern "C" void convolution_8x8x32_fl16_ver1(uint16_t* src,
                                             uint16_t* weight,
                                             int src_width_stride,
                                             int src_height_stride,
                                             int src_batch_stride,
                                             int weight_width_stride,
                                             int weight_height_stride,
                                             int kernelX,
                                             int kernelY,
                                             int stride_X_in_bytes,
                                             int stride_Y_in_bytes,
                                             int loop_srcC_kerX_iter_count); // srcC / 4 * kerX

extern "C" void convolution_8x8x32_fl16_ver1_with_offset(uint16_t* src,
                                                         uint16_t* weight,
                                                         int src_width_stride,
                                                         int src_height_stride,
                                                         int src_batch_stride,
                                                         int weight_width_stride,
                                                         int weight_height_stride,
                                                         int kernelX,
                                                         int kernelY,
                                                         int stride_X_in_bytes,
                                                         int stride_Y_in_bytes,
                                                         int loop_srcC_kerX_iter_count); // srcC / 4 * kerX

extern "C" void convolution_4x4x128_fl16_ver1(uint16_t* src,
                                              uint16_t* weight,
                                              int src_width_stride,
                                              int src_height_stride,
                                              int src_batch_stride,
                                              int weight_width_stride,
                                              int weight_height_stride,
                                              int kernelX,
                                              int kernelY,
                                              int stride_X_in_bytes,
                                              int stride_Y_in_bytes,
                                              int loop_srcC_kerX_iter_count); // srcC / 8 * kerX


extern "C" void convolution_4x4x128_fl16_ver1_with_offset(uint16_t* src,
                                                          uint16_t* weight,
                                                          int src_width_stride,
                                                          int src_height_stride,
                                                          int src_batch_stride,
                                                          int weight_width_stride,
                                                          int weight_height_stride,
                                                          int loop_srcC,// srcC / 8
                                                          int kernelY,
                                                          int stride_X_in_bytes,
                                                          int stride_Y_in_bytes,
                                                          int loop_srcC_kerX_iter_count); // srcC / 8 * kerX

extern "C" void convolution_4x8x64_fl16_ver1(uint16_t* src,
                                             uint16_t* weight,
                                             int src_width_stride,
                                             int src_height_stride,
                                             int src_batch_stride,
                                             int weight_width_stride,
                                             int weight_height_stride,
                                             int kernelX,
                                             int kernelY,
                                             int stride_X_in_bytes,
                                             int stride_Y_in_bytes,
                                             int loop_srcC_kerX_iter_count); // srcC / 4 * kerX

extern "C" void convolution_4x8x64_fl16_ver1_with_offset(uint16_t* src,
                                                         uint16_t* weight,
                                                         int src_width_stride,
                                                         int src_height_stride,
                                                         int src_batch_stride,
                                                         int weight_width_stride,
                                                         int weight_height_stride,
                                                         int loop_srcC,// srcC / 8
                                                         int kernelY,
                                                         int stride_X_in_bytes,
                                                         int stride_Y_in_bytes,
                                                         int loop_srcC_kerX_iter_count); // srcC / 4 * kerX

// страйды указаны в байтах
extern "C" void store_accums_16xX(uint16_t* dst,
                                  int width,
                                  int src_width_stride,
                                  int src_height_stride,
                                  int32_t* tic_counter,
                                  int32_t* instr_counter);
extern "C" void store_accums_16xX_relu(uint16_t* dst,
                                       int width,
                                       int src_width_stride,
                                       int src_height_stride,
                                       int32_t* tic_counter,
                                       int32_t* instr_counter);
extern "C" void store_accums_16xX_relu6(uint16_t* dst,
                                       int width,
                                       int src_width_stride,
                                       int src_height_stride,
                                       int32_t* tic_counter,
                                       int32_t* instr_counter);

// even_flag = 0 нечетное, иначе четное
extern "C" void store_accums_Xx128(uint16_t* last_line_dst, int jump_by_height, int even_h_flag, int width,
                                   int src_width_stride, int src_height_stride);

extern "C" void store_accums_Xx128_relu(uint16_t* last_line_dst, int jump_by_height, int even_h_flag, int width,
                                   int src_width_stride, int src_height_stride);

extern "C" void store_accums_Xx128_relu6(uint16_t* last_line_dst, int jump_by_height, int even_h_flag, int width,
                                   int src_width_stride, int src_height_stride);


// страйды указаны в байтах
extern "C" void store_accums_32xX(uint16_t* dst,
                                  int width,
                                  int src_width_stride,
                                  int src_height_stride);

// страйды указаны в байтах
extern "C" void store_accums_4x8x64(uint16_t* dst,
                                    int width,
                                    int src_width_stride,
                                    int src_height_stride);
extern "C" void store_accums_4x8x64_relu(uint16_t* dst,
                                         int width,
                                         int src_width_stride,
                                         int src_height_stride);
extern "C" void store_accums_4x8x64_relu6(uint16_t* dst,
                                          int width,
                                          int src_width_stride,
                                          int src_height_stride);

extern "C" void store_accums_8x8x32(uint16_t* dst,
                                    int width,
                                    int src_width_stride,
                                    int src_height_stride);
extern "C" void store_accums_8x8x32_relu(uint16_t* dst,
                                         int width,
                                         int src_width_stride,
                                         int src_height_stride);
extern "C" void store_accums_8x8x32_relu6(uint16_t* dst,
                                          int width,
                                          int src_width_stride,
                                          int src_height_stride);

// страйды указаны в байтах
extern "C" void store_accums_4Xx8Xx64(uint16_t* dst,
                                      int width,
                                      int src_width_stride,
                                      int src_height_stride,
                                      int block_w,
                                      int block_h,
                                      int jump_by_width,
                                      int jump_by_height);
extern "C" void store_accums_4Xx8Xx64_relu(uint16_t* dst,
                                           int width,
                                           int src_width_stride,
                                           int src_height_stride,
                                           int block_w,
                                           int block_h,
                                           int jump_by_width,
                                           int jump_by_height);
extern "C" void store_accums_4Xx8Xx64_relu6(uint16_t* dst,
                                            int width,
                                            int src_width_stride,
                                            int src_height_stride,
                                            int block_w,
                                            int block_h,
                                            int jump_by_width,
                                            int jump_by_height);

// страйды указаны в байтах
extern "C" void store_accums_8Xx8Xx32(uint16_t* dst,
                                      int width,
                                      int src_width_stride,
                                      int src_height_stride,
                                      int block_w,
                                      int block_h,
                                      int jump_by_width,
                                      int jump_by_height);
extern "C" void store_accums_8Xx8Xx32_relu(uint16_t* dst,
                                           int width,
                                           int src_width_stride,
                                           int src_height_stride,
                                           int block_w,
                                           int block_h,
                                           int jump_by_width,
                                           int jump_by_height);
extern "C" void store_accums_8Xx8Xx32_relu6(uint16_t* dst,
                                            int width,
                                            int src_width_stride,
                                            int src_height_stride,
                                            int block_w,
                                            int block_h,
                                            int jump_by_width,
                                            int jump_by_height);
// страйды указаны в байтах
extern "C" void store_accums_4x4x128(uint16_t* dst,
                                     int width,
                                     int src_width_stride,
                                     int src_height_stride);

extern "C" void store_accums_4x4x128_relu(uint16_t* dst,
                                          int width,
                                          int src_width_stride,
                                          int src_height_stride);
extern "C" void store_accums_4x4x128_relu6(uint16_t* dst,
                                           int width,
                                           int src_width_stride,
                                           int src_height_stride);

// страйды указаны в байтах
extern "C" void store_accums_4Xx4Xx128(uint16_t* dst,
                                       int width,
                                       int src_width_stride,
                                       int src_height_stride,
                                       int block_w,
                                       int block_h,
                                       int jump_by_height);
extern "C" void store_accums_4Xx4Xx128_relu(uint16_t* dst,
                                            int width,
                                            int src_width_stride,
                                            int src_height_stride,
                                            int block_w,
                                            int block_h,
                                            int jump_by_height);
extern "C" void store_accums_4Xx4Xx128_relu6(uint16_t* dst,
                                             int width,
                                             int src_width_stride,
                                             int src_height_stride,
                                             int block_w,
                                             int block_h,
                                             int jump_by_height);

// even_flag = 0 нечетное, иначе четное
extern "C" void store_accums_Xx64(uint16_t* last_line_dst, int jump_by_height_outside, int even_h_flag, int width,
                                  int src_width_stride, int src_height_stride, int jump_by_height_inside);

extern "C" void load_vec_regs(void* dst);
extern "C" void store_vec_regs(void* dst);
extern "C" void clear_accums();
extern "C" void add_bias_16x128(float* src);
extern "C" void add_bias_32x64(float* src);
extern "C" void add_bias_32(float* src);

void convolution_small_ker_2A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                 int kernelY, int kernelX, int dilationY, int dilationX,
                                 int strideY, int strideX, int dstH, int dstW, int group,
                                 uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                 ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
                                 Store_version st_ver, bool add_bias_flag, Conv_version conv_version = VER_4x4x128);

void init_chain_dma_convolution_small_ker_2A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                int kernelY, int kernelX, int dilationY, int dilationX,
                                                int strideY, int strideX, int dstH, int dstW, int group,
                                                int padX, int padW, int padY, int padH,
                                                uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                                ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains,
                                                uint16_t** ptr_zeros, bool add_bias_flag,
                                                uint16_t* start_adr, int local_mem_size);

void convolution_small_ker_2A2B2C_srcC_ver(
                                 uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                 int kernelY, int kernelX, int dilationY, int dilationX,
                                 int strideY, int strideX, int dstH, int dstW, int group,
                                 uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                 ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
                                 Store_version st_ver, bool add_bias_flag, float* ptr_inter_result, Conv_version conv_version = VER_4x4x128);

typedef struct Debug_data{
    int buf_dstH;
    int buf_dstW;
    int buf_srcH;
    int buf_srcW;
    int buf_srcC;
    int buf_dstC;
} Debug_data;

void init_chain_dma_convolution_small_ker_2A2B2C_srcC_ver(
                                                uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                int kernelY, int kernelX, int dilationY, int dilationX,
                                                int strideY, int strideX, int dstH, int dstW, int group,
                                                int padX, int padW, int padY, int padH,
                                                uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                                ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains,
                                                uint16_t** zeros, bool add_bias_flag,
                                                uint16_t* start_adr, int local_mem_size, float** ptr_inter_result,
                                                Debug_data tile_params);

void convolution_small_ker_1A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                 int kernelY, int kernelX, int dilationY, int dilationX,
                                 int strideY, int strideX, int dstH, int dstW, int group,
                                 uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                 ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
                                 Store_version st_ver, bool add_bias_flag);

void init_chain_dma_convolution_small_ker_1A2B2C(uint16_t* src, int batch, int srcH, int srcW, int srcC,
                                                int kernelY, int kernelX, int dilationY, int dilationX,
                                                int strideY, int strideX, int dstH, int dstW, int group,
                                                int padX, int padW, int padY, int padH,
                                                uint16_t * weight, float * bias, uint16_t * dst, int dstC,
                                                ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains,
                                                uint16_t* zeros, bool add_bias_flag);

void conv_size_selector_fl16_4x8x64(int batch, int srcH, int srcW, int srcC,
                                    int kernelY, int kernelX,
                                    int dstH, int dstW, int dstC,
                                    int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                    ConvLocalMemBufFl16* buf_sizes,
                                    int strideY, int strideX,
                                    int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size, int* offset_A);

void conv_size_selector_fl16_4x4x128(int batch, int srcH, int srcW, int srcC,
                                    int kernelY, int kernelX,
                                    int dstH, int dstW, int dstC,
                                    int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                    ConvLocalMemBufFl16* buf_sizes,
                                    int strideY, int strideX,
                                    int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size, int* offset_A);

void conv_size_selector_fl16_8x8x32(int batch, int srcH, int srcW, int srcC,
                                    int kernelY, int kernelX,
                                    int dstH, int dstW, int dstC,
                                    int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                    ConvLocalMemBufFl16* buf_sizes,
                                    int strideY, int strideX,
                                    int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size, int* offset_A);

void conv_size_selector_fl16_16x128(int batch, int srcH, int srcW, int srcC,
                                    int kernelY, int kernelX,
                                    int dstH, int dstW, int dstC,
                                    int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                    int* ptr_buf_srcH, int* ptr_buf_srcW,
                                    int* ptr_buf_dstH, int* ptr_buf_dstW, int* ptr_buf_dstC,
                                    int strideY, int strideX,
                                    int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size, int* offset_A);

void conv_size_selector_fl16_16x128(int batch, int srcH, int srcW, int srcC,
                                    int kernelY, int kernelX,
                                    int dstH, int dstW, int dstC,
                                    int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                    ConvLocalMemBufFl16* buf_sizes,
                                    int strideY, int strideX,
                                    int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size, int* offset_A);


void convolution_small_ker_4x4x128(uint16_t* src, int batch, int src_h, int src_w, int src_c,
                                  int kernel_y, int kernel_x,
                                  int stride_y, int stride_x,
                                  uint16_t * weight, float * bias, uint16_t * dst,
                                  int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver, char** ptr_buf_tics);

void convolution_small_ker_4x8x64(uint16_t* src, int batch, int src_h, int src_w, int src_c,
                                  int kernel_y, int kernel_x,
                                  int stride_y, int stride_x,
                                  uint16_t * weight, float * bias, uint16_t * dst,
                                  int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver);

void convolution_small_ker_8x8x32(uint16_t* src, int batch, int src_h, int src_w, int src_c,
                                  int kernel_y, int kernel_x,
                                  int stride_y, int stride_x,
                                  uint16_t * weight, float * bias, uint16_t * dst,
                                  int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver);

void convolution_small_ker_16x128(uint16_t* src, int batch, int src_h, int src_w, int src_c,
                                  int kernel_y, int kernel_x,
                                  int stride_y, int stride_x,
                                  uint16_t * weight, float * bias, uint16_t * dst,
                                  int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver, char** ptr_buf_tics);

void convolution_small_ker_32x64(uint16_t* src, int batch, int src_h, int src_w, int src_c,
                                 int kernel_y, int kernel_x,
                                 int stride_y, int stride_x,
                                 uint16_t * weight, float * bias, uint16_t * dst,
                                 int dst_h, int dst_w, int dst_c, bool bias_flag);

extern "C" void store_float32_block_4x4x128(float* data, int ch, int width_stride, int height_stride,
                                            int jump_by_width_0, int jump_by_width_1);

extern "C" void load_float32_block_4x4x128(float* data, int ch, int width_stride, int height_stride,
                                           int jump_by_width_0, int jump_by_width_1);

extern "C" void store_float32_block_4x8x64(float* data, int ch, int width_stride, int height_stride,
                                           int jump_by_width_0, int jump_by_width_1, int jump_by_height);

extern "C" void load_float32_block_4x8x64(float* data, int ch, int width_stride, int height_stride,
                                          int jump_by_width_0, int jump_by_width_1, int jump_by_height);

extern "C" void store_float32_block_8x8x32(float* data, int ch, int width_stride, int height_stride,
                                           int jump_by_width_0, int jump_by_width_1, int jump_by_height);
extern "C" void load_float32_block_8x8x32(float* data, int ch, int width_stride, int height_stride,
                                          int jump_by_width_0, int jump_by_width_1, int jump_by_height);

void convolution_small_ker_4x4x128_srcC_ver(
                    uint16_t* src, int batch, int src_h, int src_w, int src_c,
                    int kernel_y, int kernel_x,
                    int stride_y, int stride_x,
                    uint16_t * weight, float * bias, uint16_t * dst,
                    int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver,
                    int load_format,
                    int store_format,
                    bool start_clear_acc,
                    int tile_size_dstC
                    );

void convolution_small_ker_4x8x64_srcC_ver(
                    uint16_t* src, int batch, int src_h, int src_w, int src_c,
                    int kernel_y, int kernel_x,
                    int stride_y, int stride_x,
                    uint16_t * weight, float * bias, uint16_t * dst,
                    int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver,
                    int load_format, // 0 - без загузки, 1 - загрузка float32
                    int store_format, // 2 - store float32, 1 - store float16, 0 - без выгрузки
                    bool start_clear_acc,
                    int tile_size_dstC
                    );

void convolution_small_ker_8x8x32_srcC_ver(
                    uint16_t* src, int batch, int src_h, int src_w, int src_c,
                    int kernel_y, int kernel_x,
                    int stride_y, int stride_x,
                    uint16_t * weight, float * bias, uint16_t * dst,
                    int dst_h, int dst_w, int dst_c, bool bias_flag, int offset_A, Store_version st_ver,
                    int load_format, // 0 - без загузки, 1 - загрузка float32
                    int store_format, // 2 - store float32, 1 - store float16, 0 - без выгрузки
                    bool start_clear_acc,
                    int tile_size_dstC
                    );


void conv_size_selector_fl16_4x4x128_srcC(int batch, int srcH, int srcW, int srcC,
                                         int kernelY, int kernelX,
                                         int dstH, int dstW, int dstC,
                                         int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,
                                         ConvLocalMemBufFl16* buf,
                                         int strideY, int strideX,
                                         int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size,
                                         int* offset_A);

void conv_size_selector(int batch, int srcH, int srcW, int srcC,
                        int kernelY, int kernelX,
                        int dstH, int dstW, int dstC,
                        /*int* ptr_tile_padX, int* ptr_tile_padY, int* ptr_tile_padW, int* ptr_tile_padH,*/
                        ConvLocalMemBufFl16* buf,
                        int strideY, int strideX,
                        int count_buf_A, int count_buf_B, int count_buf_C, bool bias_flag, int local_mem_size,
                        int* offset_A);

void init_chain_dma_convolution_small_ker_2A2B2C_srcC_ver_with_dilation(
        Tensor_fl16* input_tensor,
        //uint16_t* src, int batch, int srcH, int srcW, int srcC,
        Weight_fl16* input_weight,
        Tensor_fl16* output_tensor,
        int group,
        int global_zero_padX, int global_zero_padW, int global_zero_padY,
        int global_zero_padH, float * bias,
        ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains,
        uint16_t** ptr_zeros, bool add_bias_flag, uint16_t* start_adr,
        int local_mem_size, float** ptr_inter_result, Debug_data tile_params);

void convolution_small_ker_2A2B2C_srcC_ver_with_dilation(
        Tensor_fl16* input_tensor,
        //uint16_t* src, int batch, int srcH, int srcW, int srcC,
        Weight_fl16* input_weight,
        Tensor_fl16* output_tensor,
        int group,
        float * bias,
        ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
        Store_version st_ver, bool add_bias_flag, float* ptr_inter_result);

void convolution_small_ker_4x4x128_with_dilation(
        Tensor_fl16* input_tensor,
        Weight_fl16* input_weight,
        float * bias, Tensor_fl16* output_tensor,
        bool bias_flag, int offset_A, Store_version st_ver,
        bool clear_accum, bool store_result,
        char** ptr_buf_tics_out);

void init_chain_dma_convolution_small_ker_2A2B2C_srcC_ver_with_dilation_tmp_res(
        Tensor_fl16* input_tensor,
        //uint16_t* src, int batch, int srcH, int srcW, int srcC,
        Weight_fl16* input_weight,
        Tensor_fl16* output_tensor,
        int group,
        int global_zero_padX, int global_zero_padW, int global_zero_padY,
        int global_zero_padH, float * bias,
        ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains,
        uint16_t** ptr_zeros, bool add_bias_flag, uint16_t* start_adr,
        int local_mem_size, float** ptr_inter_result, Debug_data tile_params);

void convolution_small_ker_2A2B2C_srcC_ver_with_dilation_tmp_res(
        Tensor_fl16* input_tensor,
        //uint16_t* src, int batch, int srcH, int srcW, int srcC,
        Weight_fl16* input_weight,
        Tensor_fl16* output_tensor,
        int group,
        float * bias,
        ConvLocalMemBufFl16* buffer, MatMulDmaChains4* chains, uint16_t* zeros,
        Store_version st_ver, bool add_bias_flag, float* ptr_inter_result);

void convolution_small_ker_4x4x128_with_dilation_tmp_res(
        Tensor_fl16* input_tensor,
        Weight_fl16* input_weight,
        float * bias, Tensor_fl16* output_tensor,
        bool bias_flag, int offset_A, Store_version st_ver,
        // clear_accum:  0 - nop, 1 - clear accs, 2 - load float32
        // store_result: 0 - nop, 1 - store float16, 2 - store float32
        int clear_accum, int store_result,
        char** ptr_buf_tics_out);

#endif
